
/* Network traffic analyzer
 * (c) 2001, Emil Mikulic.
 */

/* Host database */

#include "host_db.h"
#include "bignum.h"



/* Global database */
db host_db;



void host_db_init(void)
{
	db_init(&host_db);
}

void host_db_free(void)
{
	unsigned int i;

	for (i=0; i<host_db.used; i++)
		free(host_db.records[i]);

	db_free(&host_db);
}



/* host_record needs to be deallocated when it's no longer needed */
host_record *host_make(const dword ip_addr)
{
	host_record *h = (host_record*)malloc(sizeof(host_record));

	h->ip_addr = ip_addr;
	h->hostname = NULL;
	h->data_in.hi = h->data_in.lo =
		h->data_out.hi = h->data_out.lo = 0;

	return h;
}



/* update transfer amount */
__inline void host_update_in(host_record *h, const dword in)
{
	i64 add = {0, in};
	add_64bit(&(h->data_in), add);
}

__inline void host_update_out(host_record *h, const dword out)
{
	i64 add = {0, out};
	add_64bit(&(h->data_out), add);
}



/* add or update a host, depending on whether it exists in the db or not */
__inline host_record *host_from_ip(const dword ip)
{
	int i;
	host_record *h;
	extern dword local_ip;

	assert(host_db.used > 0);

	/* note: assumes that first record is local */
	if (ip == local_ip) return (host_record*)db_get(host_db, 0);	

	h = NULL;
	for (i=host_db.used-1; i>=0; i--)
	{
		host_record *tmp = (host_record*)db_get(host_db, i);
		if (tmp->ip_addr == ip) h = tmp;
	}

	if (!h)
	{
		h = host_make(ip);
		if (!db_add(&host_db, h)) freakout();
	}

	return h;
}



__inline void host_transfer_in(const dword ip, const dword in)
{
	host_update_in(host_from_ip(ip), in);
}

__inline void host_transfer_out(const dword ip, const dword out)
{
	host_update_out(host_from_ip(ip), out);
}



__inline unsigned int ip_from_quad(const int a, const int b, const int c, const int d)
{
	return (
			((a & 255) << 24) |
			((b & 255) << 16) |
			((c & 255) << 8) |
			(d & 255)
		);
}



/* note: buffer needs to be deallocated */
__inline char *quad_from_ip(const dword ip)
{
	char *buf = (char*)malloc(16);

	sprintf(buf,"%d.%d.%d.%d",
			(ip >> 24) & 255,
			(ip >> 16) & 255,
			(ip >> 8) & 255,
			(ip & 255)
		);

	return buf;
}


void host_db_dump(FILE *fp)
{
	int i;

	fprintf(fp, "%u\n", host_db.used);
	for (i=0; i<host_db.used; i++)
	{
		host_record *h = (host_record*)db_get(host_db, i);
		
		fprintf(fp, "%u|%s|%u:%u|%u:%u\n",
			h->ip_addr,
			(h->hostname)?h->hostname:"",
			h->data_in.hi,
			h->data_in.lo,
			h->data_out.hi,
			h->data_out.lo);
	}
}

